

float4x4 tWI: WORLDINVERSE;        //the models world matrix
float4x4 tV: VIEW;         //view matrix as set via Renderer (EX9)
float4x4 tP: PROJECTION;   //projection matrix as set via Renderer (EX9)
float4x4 tWVP: WORLDVIEWPROJECTION;

float3 LightDir;

//Colors
float4 AmbientColor : COLOR <String uiname="Ambient";>  = {0.1, 0.1, 0.1, 1};
float4 DiffuseColor : COLOR <String uiname="Diffuse";>  = {0, 0.5, 1, 1};
float4 SpecularColor : COLOR <String uiname="Specular";>  = {1, 1, 1, 1};

float SpecP;

//Texture
texture Tex;
sampler Samp = sampler_state
{
   Texture   = <Tex>;
   MinFilter = Linear;
   MagFilter = Linear;
   MipFilter = Linear;   
   AddressU  = Clamp;
   AddressV  = Clamp;
};

//Data
struct vs2ps
{
       float4 Pos	: POSITION;
       float2 TexCd	: TEXCOORD0;
       float3 L	        : TEXCOORD1;
       float3 N	        : TEXCOORD2;
       float3 V	        : TEXCOORD3;
};
// --------------------------------------------------------------------------------------------------
// VERTEXSHADERS
// --------------------------------------------------------------------------------------------------

vs2ps VS(

      float4 Pos    : POSITION,
      float2 TexCd  : TEXCOORD,
      float3 N      : NORMAL)
    
{
	vs2ps Out = (vs2ps) 0;

        Out.Pos = mul(Pos, tWVP);
        
        Out.TexCd = TexCd;
        
        Out.L = normalize(LightDir);
        Out.N = normalize(mul(tWI, N));
        Out.V = LightDir - Pos;
	
        return Out;
}

// --------------------------------------------------------------------------------------------------
// PIXELSHADERS:
// --------------------------------------------------------------------------------------------------

float4 PS(vs2ps In) : COLOR
{

        float3 ViewDir = normalize(In.V);
	
        float4 Col = tex2D(Samp, In.TexCd);
        float Diff = saturate(dot(In.L, In.N));

        float3 Refl = normalize(2 * Diff * In.N - In.L);
        float Specular = pow(saturate(dot(Refl, ViewDir)), SpecP);


        return (Col  * AmbientColor) + (Col * DiffuseColor * Diff )+ (SpecularColor * Specular);

}

technique SpecLight
{
	pass P0
	{
                //ZEnable = TRUE;
		//ZWriteEnable = TRUE;
		//AlphaBlendEnable = FALSE;
		
		VertexShader = compile vs_2_0 VS();
		PixelShader = compile ps_2_0 PS();
	}
}
