float4x4 tW: WORLD;        //the models world matrix
float4x4 tV: VIEW;         //view matrix as set via Renderer (EX9)
float4x4 tP: PROJECTION;   //projection matrix as set via Renderer (EX9)
float4x4 tWVP: WORLDVIEWPROJECTION;

//Distance for absorption factor
float Du = 2.0f;

//Consistency of the transmitter
float C = 12.0f;

float Alpha = 1.0f;

//Texture

texture BGScene;
sampler BGSceneSampler = sampler_state
{
   Texture = <BGScene>;
   MinFilter = Linear;
   MagFilter = Linear;
   MipFilter = Linear;
   AddressU  = Clamp;
   AddressV  = Clamp;
};

texture Scene;
sampler SceneSampler = sampler_state
{
   Texture = <Scene>;
   MinFilter = Linear;
   MagFilter = Linear;
   MipFilter = Linear;
   AddressU  = Clamp;
   AddressV  = Clamp;
};

texture Depth1;
sampler Depth1Sampler = sampler_state
{
   Texture = <Depth1>;
   MinFilter = Linear;
   MagFilter = Linear;
   MipFilter = Linear;   
   AddressU  = Clamp;
   AddressV  = Clamp;
};

texture Depth2;
sampler Depth2Sampler = sampler_state
{
   Texture = <Depth2>;
   MinFilter = Linear;
   MagFilter = Linear;
   MipFilter = Linear;   
   AddressU  = Clamp;
   AddressV  = Clamp;
};



float4x4 tTex: TEXTUREMATRIX <string uiname="Texture Transform";>;

//Data

struct vs2ps
{
    float4 Pos : POSITION;
    float4 TexCd : TEXCOORD0;
};

// --------------------------------------------------------------------------------------------------
// VERTEXSHADERS
// --------------------------------------------------------------------------------------------------

vs2ps VS(
    float4 Pos : POSITION,
    float4 TexCd : TEXCOORD0)
{
    //Inititalize all fields of output struct with 0
    vs2ps Out = (vs2ps)0;

    //Transform position
    Out.Pos = mul(Pos, tWVP);

    //Transform texturecoordinates
    Out.TexCd = mul(TexCd, tTex);

    return Out;
}

// --------------------------------------------------------------------------------------------------
// PIXELSHADERS
// --------------------------------------------------------------------------------------------------

float4 PS(float2 Tex: TEXCOORD0) : COLOR
{
        //Scene
        float4 Col=tex2D(SceneSampler, Tex);
        
        //BackGround
        float4 BGColor=tex2D(BGSceneSampler, Tex);
	
	//Depth1 & Depth2
        float depth1=tex2D(Depth1Sampler, Tex).r;
        float depth2=tex2D(Depth2Sampler, Tex).r;
        
        //Object thickness
        float distance = abs(depth2 - depth1) ;
	
	//Absorbtion factor
        float3 a;
	       a.r = (-log(Col.r))/Du ;
	       a.g = (-log(Col.g))/Du;
	       a.b = (-log(Col.b))/Du;
	       a = a ;
	
        //Transmittance factor
        float4 T;
	       T.r = exp((-a.r) * C * distance);
	       T.g = exp((-a.g) * C * distance);
	       T.b = exp((-a.b) * C * distance);
	       T.a = Alpha;


        return T * BGColor;
}

technique Final
{
	pass P0
	{
	
	//ZEnable = TRUE;
	//ZWriteEnable = TRUE;
	AlphaBlendEnable = TRUE ;
	
        VertexShader = compile vs_1_1 VS();
        PixelShader = compile ps_2_0 PS();
	}
}
